@@ -1,7 +1,11 @@
-#################################################################################################
-# This file is part of RobustGaussianFittingLibrary, a free library WITHOUT ANY WARRANTY        # 
-# Copyright: 2019-2020 Deutsches Elektronen-Synchrotron                                         # 
-#################################################################################################
+"""
+------------------------------------------------------
+This file is part of RobustGaussianFittingLibrary,
+a free library WITHOUT ANY WARRANTY
+Copyright: 2017-2020 LaTrobe University Melbourne,
+           2019-2020 Deutsches Elektronen-Synchrotron
+------------------------------------------------------
+"""
 
 """ A ctypes wrapper for the Robust Gaussian Fitting Library C file
 Nothing to look for in this file, its just a wrapper
@@ -58,27 +62,57 @@ RGFCLib.MSSEWeighted.argtypes = [
                 ctypes.c_uint, ctypes.c_float, ctypes.c_uint, ctypes.c_float ]
 
 '''
-void RobustSingleGaussianVec(float *vec, float *modelParams, float theta, unsigned int N,
-		float topkPerc, float botkPerc, float MSSE_LAMBDA, unsigned char optIters, float minimumResidual)
+void fitValue(float* inVec,
+			  float* inWeights,
+			  float* modelParams,
+			  float theta,
+			  unsigned int inN,
+              float topkPerc,
+			  float botkPerc,
+              float MSSE_LAMBDA,
+			  unsigned char optIters,
+              float minimumResidual,
+			  unsigned int downSampledSize);
 '''
-RGFCLib.RobustSingleGaussianVec.argtypes = [
+RGFCLib.fitValue.argtypes = [
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
-                ctypes.c_float, ctypes.c_int, ctypes.c_float, 
-                ctypes.c_float, ctypes.c_float, ctypes.c_uint8, ctypes.c_float]
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                ctypes.c_float, 
+                ctypes.c_int, 
+                ctypes.c_float, 
+                ctypes.c_float, 
+                ctypes.c_float, 
+                ctypes.c_uint8, 
+                ctypes.c_float, 
+                ctypes.c_int]
 
 '''
-void fitValue2Skewed(float *vec, float *weights, 
-					float *modelParams, float theta, unsigned int N,
-					float topkPerc, float botkPerc,
-					float MSSE_LAMBDA, unsigned char optIters, float minimumResidual);
+void fitValue2Skewed(float* inVec,
+			         float* inWeights,
+			         float* modelParams,
+			         float theta,
+			         unsigned int inN,
+					 float topkPerc,
+			         float botkPerc,
+                     float MSSE_LAMBDA,
+			         unsigned char optIters,
+                     float minimumResidual,
+			         unsigned int downSampledSize);
 '''                    
 RGFCLib.fitValue2Skewed.argtypes = [
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
-                ctypes.c_float, ctypes.c_uint32, ctypes.c_float, 
-                ctypes.c_float, ctypes.c_float, ctypes.c_uint8, ctypes.c_float]                
+                ctypes.c_float, 
+                ctypes.c_int,
+                ctypes.c_float, 
+                ctypes.c_float, 
+                ctypes.c_float, 
+                ctypes.c_uint8, 
+                ctypes.c_float, 
+                ctypes.c_int]
+
 
 '''
 void medianOfFits(float *vec, float *weights, 
@@ -119,18 +153,19 @@ RGFCLib.RobustAlgebraicLineFittingTensor.argtypes = [
                 ctypes.c_float, ctypes.c_float, ctypes.c_float]                
                 
 '''
-void RobustSingleGaussianTensor(float *inTensor, unsigned char* inMask,
-				float *modelParamsMap, unsigned int N, unsigned int X, unsigned int Y,
-				float topkPerc, float botkPerc, float MSSE_LAMBDA, 
-				unsigned char optIters, float minimumResidual)
+void fitValueTensor(float* inTensor, float* inWeights, float* modelParamsMap,
+					unsigned int N, unsigned int X, unsigned int Y,
+					float topkPerc, float botkPerc, float MSSE_LAMBDA,
+					unsigned char optIters, float minimumResidual,
+					unsigned int downSampledSize);
 '''
-RGFCLib.RobustSingleGaussianTensor.argtypes = [
+RGFCLib.fitValueTensor.argtypes = [
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
-                np.ctypeslib.ndpointer(ctypes.c_uint8, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32,
                 ctypes.c_float, ctypes.c_float, ctypes.c_float, 
-                ctypes.c_uint8, ctypes.c_float]
+                ctypes.c_uint8, ctypes.c_float, ctypes.c_uint32]
 
 '''
 void RobustAlgebraicPlaneFitting(float* x, float* y, float* z, float* mP, float* mP_Init,
@@ -210,4 +245,34 @@ RGFCLib.fitBackgroundRadially.argtypes = [
                 ctypes.c_uint8, ctypes.c_uint32, 
                 ctypes.c_uint32, ctypes.c_uint32, 
                 ctypes.c_float, ctypes.c_float, 
-                ctypes.c_float, ctypes.c_uint8, ctypes.c_float]
\ No newline at end of file
+                ctypes.c_float, ctypes.c_uint8, ctypes.c_float]
+
+'''
+void fitBackgroundCylindrically(float* inTensor,
+								unsigned char* inMask,
+                                float* modelParamsMap,
+								float* vecMP,
+                                unsigned int minRes,
+                                unsigned int maxRes,
+                                unsigned int shellWidth,
+                                unsigned char includeCenter,
+                                unsigned int finiteSampleBias,
+								unsigned int N,
+                                unsigned int X,
+								unsigned int Y,
+                                float topkPerc,
+								float botkPerc,
+                                float MSSE_LAMBDA,
+                                unsigned char optIters,
+						        float minimumResidual)
+'''
+RGFCLib.fitBackgroundCylindrically.argtypes = [
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                np.ctypeslib.ndpointer(ctypes.c_uint8, flags='C_CONTIGUOUS'),
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32, 
+                ctypes.c_uint8, ctypes.c_uint32, 
+                ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32, 
+                ctypes.c_float, ctypes.c_float, 
+                ctypes.c_float, ctypes.c_uint8, ctypes.c_float]
