@@ -1,7 +1,11 @@
-#################################################################################################
-# This file is part of RobustGaussianFittingLibrary, a free library WITHOUT ANY WARRANTY        # 
-# Copyright: 2019-2020 Deutsches Elektronen-Synchrotron                                         # 
-#################################################################################################
+"""
+------------------------------------------------------
+This file is part of RobustGaussianFittingLibrary,
+a free library WITHOUT ANY WARRANTY
+Copyright: 2017-2020 LaTrobe University Melbourne,
+           2019-2020 Deutsches Elektronen-Synchrotron
+------------------------------------------------------
+"""
 
 import RobustGaussianFittingLibrary 
 import RobustGaussianFittingLibrary.useMultiproc
@@ -10,6 +14,7 @@ import RobustGaussianFittingLibrary.basic
 import numpy as np
 import time
 import matplotlib.pyplot as plt
+from mpl_toolkits.mplot3d import Axes3D
 import os
 import scipy.stats
 from cProfile import label
@@ -180,14 +185,16 @@ def test_removeIslands():
 def test_bigTensor2SmallsInds():
     print('test_bigTensor2SmallsInds')
     a = (100*np.random.randn(20,16,11)).astype('int')
-    rowClmInds, segInds = RobustGaussianFittingLibrary.useMultiproc.bigTensor2SmallsInds(a.shape, 2,3)
+    rowClmInds, segInds = \
+        RobustGaussianFittingLibrary.useMultiproc.bigTensor2SmallsInds(a.shape, 
+                                                                       2, 3)
     print(rowClmInds)
 
 def test_fitValue_sweep():
     print('test_fitValue_sweep_over_N')
     numIter = 1000
-    maxN = 100
-    minN = 5
+    maxN = 400 
+    minN = 200
     mean_inliers = np.zeros((maxN-minN, numIter))
     std_inliers = np.zeros((maxN-minN, numIter))
     robust_mean = np.zeros((maxN-minN, numIter))
@@ -203,11 +210,13 @@ def test_fitValue_sweep():
             testData = np.concatenate((RNN0, RNN1)).flatten()
             np.random.shuffle(testData)
             time_time = time.time()
-            rmode, rstd = RobustGaussianFittingLibrary.fitValue(testData, 
+            rmode, rstd = RobustGaussianFittingLibrary.fitValue(testData,
+                                                                fit2Skewed = False, 
                                                                 topKthPerc=0.5, 
                                                                 bottomKthPerc=0.3,
                                                                 MSSE_LAMBDA=3.0,
-                                                                optIters= 1)
+                                                                optIters= 10,
+                                                                downSampledSize = 100)
             timeR = time.time() - time_time
             mean_inliers[N-minN, iter] = RNN0.mean()
             std_inliers[N-minN, iter] = RNN0.std()
@@ -329,7 +338,7 @@ def test_fitValueVSMeanShiftPy():
     fig.legend()
     plt.show()
     
-    mP = RobustGaussianFittingLibrary.fitValue2Skewed(inZ)
+    mP = RobustGaussianFittingLibrary.fitValue(inZ, fit2Skewed = False)
     print(mP)
     Xax = np.arange(inX.min(), inX.max())
     Yax = np.arange(inY.min(), inY.max())
@@ -635,10 +644,15 @@ def test_fitPlaneVSMeanShiftPy():
 
 def test_RobustAlgebraicLineFittingPy():
     print('test_RobustAlgebraicLineFittingPy')
-    n_in = 100
     inSigma = 3
+    slope = 0.5
+    intercept = 10
+    print('slope: ', slope)
+    print('intercept: ', intercept)
+    print('Scale: ', inSigma)
+    n_in = 100
     inX = 200*(np.random.rand(n_in)-0.5)
-    inY = 0.5*inX + 10 + inSigma*np.random.randn(n_in)
+    inY = slope*inX + intercept + inSigma*np.random.randn(n_in)
     n_out = 80
     outX = 200*(np.random.rand(n_out)-0.5)
     outY = 200*(np.random.rand(n_out)-0.25)
@@ -712,7 +726,62 @@ def test_fitBackground():
     axes[2].set_ylim([winYL, winYU])
     fig.colorbar(im2, ax=axes[2], shrink = 0.5)
     plt.show()
+    
+def centeredDistanceMatrix(n):
+    n = int(2*int(n/2) + 1)
+    x,y = np.meshgrid(range(n),range(n))
+    return np.sqrt((x-(n/2)+1)**2+(y-(n/2)+1)**2)
 
+def test_fitBackgroundCylindrically():
+    print('test_fitBackgroundCylindrically')
+    n_F = 50
+    n_R = 1024
+    n_C = 1024
+    imDiag = (n_R**2 + n_C**2)**0.5
+    radii = centeredDistanceMatrix(imDiag)
+    water_avg_profile = np.exp(-(np.arange(imDiag)-n_R/4)**2/(2*(n_R/16)**2))
+    water2D_avg = np.exp(-(radii-n_R/4)**2/(2*(n_R/16)**2))
+    water2D_std = water2D_avg**0.5
+    r_cent = int(water2D_std.shape[0]/2)
+    c_cent = int(water2D_std.shape[1]/2)
+    water2D_avg = water2D_avg[r_cent - int(n_R/2):r_cent + int(n_R/2),
+                                          r_cent - int(n_C/2):r_cent + int(n_C/2)]
+    water2D_std = water2D_std[r_cent - int(n_R/2):r_cent + int(n_R/2),
+                                          r_cent - int(n_C/2):r_cent + int(n_C/2)]
+    normalNoisePattern = np.zeros((n_F, n_R, n_C), dtype='float32')
+    for fCnt in range(n_F):
+        normalNoisePattern[fCnt] = \
+            (np.random.randn(n_R, n_C) + water2D_avg)*water2D_std
+    
+    print(normalNoisePattern.shape)
+    mP, est_profile = \
+        RobustGaussianFittingLibrary.basic.fitBackgroundCylindrically(\
+        inTensor = normalNoisePattern, 
+        inMask = None,
+        minRes = 1,
+        includeCenter = 0,
+        maxRes = None,
+        shellWidth = 3,
+        topKthPerc = 0.5,
+        bottomKthPerc = 0.25,
+        MSSE_LAMBDA = 3.0,
+        optIters = 16,
+        numStrides = 2,
+        finiteSampleBias = 10000,
+        minimumResidual = 0,
+        return_vecMP = True)
+
+    plt.figure(1)
+    plt.imshow(normalNoisePattern[0])
+    plt.figure(2)
+    plt.imshow(mP[0])
+    plt.figure(3)
+    plt.imshow(mP[1])
+    plt.figure(4)
+    plt.plot(est_profile[0], label = 'ext_profile')
+    plt.plot(water_avg_profile, label = 'true_profile')
+    plt.show()
+    
 def test_fitBackgroundRadially():
     print('test_fitBackgroundRadially')
     XSZ = 1024
@@ -848,7 +917,11 @@ def test_SginleGaussianVec():
     testData = np.concatenate((RNN0, RNN1)).flatten()
     np.random.shuffle(testData)
     print('testing RobustSingleGaussianVecPy')
-    mP = RobustGaussianFittingLibrary.fitValue(testData, topKthPerc = 0.5, bottomKthPerc=0.35, MSSE_LAMBDA=3.0)
+    mP = RobustGaussianFittingLibrary.fitValue(testData, 
+                                               fit2Skewed = False,
+                                               topKthPerc = 0.5, 
+                                               bottomKthPerc=0.35, 
+                                               MSSE_LAMBDA=3.0)
     print(mP)
     RobustGaussianFittingLibrary.misc.naiveHist(testData, mP)
     plt.plot(testData,'.'), plt.show()
@@ -866,7 +939,11 @@ def test_fitValue2Skewed():
     testData = np.concatenate((RNN0, RNN1)).flatten()
     np.random.shuffle(testData)
     print('testing fitValue2Skewed')
-    mP = RobustGaussianFittingLibrary.fitValue2Skewed(testData, topKthPerc = 0.43, bottomKthPerc=0.37, MSSE_LAMBDA=3.0)
+    mP = RobustGaussianFittingLibrary.fitValue(testData, 
+                                               fit2Skewed = True,
+                                               topKthPerc = 0.43, 
+                                               bottomKthPerc=0.37, 
+                                               MSSE_LAMBDA=3.0)
     RobustGaussianFittingLibrary.misc.naiveHist(testData, mP)
     plt.plot(testData,'.'), plt.show()
     plt.plot(testData,'.'), 
@@ -879,7 +956,7 @@ def test_fitValue2Skewed():
 def test_fitValue2Skewed_sweep_over_N():
     print('test_fitValue2Skewed_sweep_over_N')
     numIter = 1000
-    maxN = 64
+    maxN = 30
     minN = 3
     mean_inliers = np.zeros((maxN-minN, numIter))
     std_inliers = np.zeros((maxN-minN, numIter))
@@ -887,30 +964,28 @@ def test_fitValue2Skewed_sweep_over_N():
     robustSkew_std = np.zeros((maxN-minN, numIter))
     robust_mean = np.zeros((maxN-minN, numIter))
     robust_std = np.zeros((maxN-minN, numIter))
-    pBar = RobustGaussianFittingLibrary.misc.textProgBar(maxN-minN)
     x = np.zeros(maxN-minN)
-    
     timeSkew = 0
     timeR = 0
+    pBar = RobustGaussianFittingLibrary.misc.textProgBar(maxN-minN)
     for N in range(minN,maxN):
         for iter in range(numIter):
             RNN0 = np.random.randn(N)
-            RNN1 = 7+5*(np.random.rand(int(N*0.25))-0.5)
+            RNN1 = 12+3*(np.random.rand(int(N*0.5))-0.5)
             testData = np.concatenate((RNN0, RNN1)).flatten()
             np.random.shuffle(testData)
             time_time = time.time()
-            rmodeSkew, rstdSkew = RobustGaussianFittingLibrary.fitValue2Skewed(testData, 
-                                                                topKthPerc=0.5, 
-                                                                bottomKthPerc=0.3,
-                                                                MSSE_LAMBDA=3.0,
-                                                                optIters= 12)
+            rmodeSkew, rstdSkew = \
+                RobustGaussianFittingLibrary.fitValue(
+                    testData,
+                    topKthPerc = 0.5,
+                    bottomKthPerc = 0.45,
+                    optIters = 12,
+                    fit2Skewed = True)
             timeSkew = time.time() - time_time
             time_time = time.time()
-            rmode, rstd = RobustGaussianFittingLibrary.fitValue(testData, 
-                                                                topKthPerc=0.5, 
-                                                                bottomKthPerc=0.3,
-                                                                MSSE_LAMBDA=3.0,
-                                                                optIters= 12)
+            rmode, rstd = RobustGaussianFittingLibrary.fitValue(testData,
+                                                                fit2Skewed = False)
             timeR = time.time() - time_time
             mean_inliers[N-minN, iter] = RNN0.mean()
             std_inliers[N-minN, iter] = RNN0.std()
@@ -951,12 +1026,17 @@ def test_flatField():
 
     modelCnt = 0
     mP = RobustGaussianFittingLibrary.fitValue(testData, 
-                            topKthPerc = 0.49, bottomKthPerc=0.45, MSSE_LAMBDA=2.0)
+                            topKthPerc = 0.49, bottomKthPerc=0.45, 
+                            fit2Skewed = False, MSSE_LAMBDA=2.0)
     RobustGaussianFittingLibrary.misc.naiveHist(data, mP)
 
 
     for modelCnt in range(4):
-        mP = RobustGaussianFittingLibrary.fitValue(testData, topKthPerc = 0.49, bottomKthPerc=0.45, MSSE_LAMBDA=1.0)
+        mP = RobustGaussianFittingLibrary.fitValue(testData, 
+                                                   fit2Skewed = False,
+                                                   topKthPerc = 0.49, 
+                                                   bottomKthPerc=0.45, 
+                                                   MSSE_LAMBDA=1.0)
         probs = np.random.rand(testData.shape[0]) - np.exp(-(testData - mP[0])**2/(2*mP[1]**2))
         probs[testData<mP[0]] = 0
         probs[probs>mP[0]+3.0*mP[1]] = 1
@@ -967,7 +1047,7 @@ def test_flatField():
     RobustGaussianFittingLibrary.misc.sGHist_multi_mP(data, mP_All, SNR=2.5)
     
 def test_fitValueTensor_MultiProc():
-    print('test_fitValueTensor_MultiProc')
+    print('fitValueTensor functions')
     SIGMA = 10
     RNN1 = SIGMA*np.random.randn(500-50-3, 18, 38)
     RNN2 = 5*SIGMA + 5*SIGMA*np.random.randn(50, 18, 38)
@@ -976,10 +1056,9 @@ def test_fitValueTensor_MultiProc():
     testData = np.concatenate((RNN1, RNN2))
     testData = np.concatenate((testData, RNU))
     
-    inMask = np.ones(testData.shape, dtype='uint8')
-    #inMask[testData < 3] = 0
+    inMask = np.ones(testData.shape)
     
-    print('testing RobustSingleGaussianTensorPy')
+    print('testing fitValueTensor')
     nowtime = time.time()
     modelParamsMap = RobustGaussianFittingLibrary.fitValueTensor(testData, inMask)
     print(time.time() - nowtime)
@@ -987,9 +1066,11 @@ def test_fitValueTensor_MultiProc():
     
     print('testing fitValueTensor_MultiProc')
     nowtime = time.time()
-    modelParamsMap = RobustGaussianFittingLibrary.useMultiproc.fitValueTensor_MultiProc(testData, inMask,
-                                                           numRowSegs = 6,
-                                                           numClmSegs = 12)
+    modelParamsMap = RobustGaussianFittingLibrary.useMultiproc.fitValueTensor_MultiProc(
+        testData, 
+        inMask,
+        numRowSegs = 6,
+        numClmSegs = 12)
     print(time.time() - nowtime)
     print(modelParamsMap)
 
@@ -1020,13 +1101,25 @@ def test_fitValueSmallSample():
     testData = np.hstack((inliers, outliers))
     np.random.shuffle(testData)
     print('testing fitValue with ' + str(inliers.shape[0]) + ' inliers and ' + str(outliers.shape[0]) + ' outliers.')
-    mP = RobustGaussianFittingLibrary.fitValue(testData, topKthPerc = 0.5, 
-                                               bottomKthPerc=0.35, MSSE_LAMBDA=3.0,
+    mP = RobustGaussianFittingLibrary.fitValue(testData, fit2Skewed = False,
                                                modelValueInit = 100)
     print('inliers mean ' + str(inliers.mean()) + ' inliers std ' + str(inliers.std()))
     print(mP)
 
-def test_fit2Poisson():    
+def test_fitValue2SkewedSmallSample(): 
+    print('test_fitValue2SkewedSmallSample')
+    inliers = np.random.randn(100)
+    outliers = np.array([100, 64])
+    testData = np.hstack((inliers, outliers))
+    np.random.shuffle(testData)
+    print('testing fitValue with ' + str(inliers.shape[0]) + ' inliers and ' + str(outliers.shape[0]) + ' outliers.')
+    mP = RobustGaussianFittingLibrary.fitValue(testData, 
+                                               fit2Skewed = True, bottomKthPerc = 0.4, 
+                                                      modelValueInit = 100, optIters = 12)
+    print('inliers mean ' + str(inliers.mean()) + ' inliers std ' + str(inliers.std()))
+    print(mP)
+
+def test_fit2Poisson():
     #import scipy.math.factorial as factorial
     numIters = 20
     photon = 73.5
@@ -1074,7 +1167,8 @@ def test_fit2Poisson():
             SNRs_true = (((outliers - inliers_mu)/inliers_std) >= minSNR).sum()/numOutliers
             vec_contaminated = np.hstack((vec.copy(), outliers))
         
-            _mP[iters] = RobustGaussianFittingLibrary.fitValue(vec_contaminated, 
+            _mP[iters] = RobustGaussianFittingLibrary.fitValue(vec_contaminated,
+                                                               fit2Skewed = False,
                                                         minimumResidual = 0.2 * photon, MSSE_LAMBDA = 4.0)
             _meanShift[iters] = RobustGaussianFittingLibrary.basic.fitValue_by_meanShift(vec_contaminated, minSNR = 6.0)
         _mP = _mP.mean(0)
@@ -1123,7 +1217,7 @@ def test_fit2Poisson():
     plt.show()    
 
 def test_medianOfFits():
-    print('test_fitValueSmallSample')
+    print('test_medianOfFits')
     inliers = np.random.randn(100)
     outliers = np.array([100, 64])
     testData = np.hstack((inliers, outliers))
@@ -1134,7 +1228,6 @@ def test_medianOfFits():
                                                    topkMax = 0.7,
                                                    topkMin = 0.3,
                                                    numSamples = 50,
-                                                   samplePerc = 0.1,
                                                    MSSE_LAMBDA = 3.0,
                                                    modelValueInit = 0,
                                                    optIters = 12,
@@ -1142,8 +1235,42 @@ def test_medianOfFits():
     print('inliers mean ' + str(inliers.mean()) + ' inliers std ' + str(inliers.std()))
     print(mP)    
 
+def test_getTriangularVertices():
+    RobustGaussianFittingLibrary.misc.getTriangularVertices(
+        n = 1000,
+        phi_start = 0,
+        phi_end = np.pi,
+        plotIt = True)
+
+
+def myFunc(anInput):
+    data, op_type = anInput
+    if(op_type=='mean'):
+        to_return = data.mean()
+    if(op_type=='max'):
+        to_return = data.max()
+    return(np.array([to_return]))
+    
+def test_multiprocessor():
+    N = 10000
+    Data = np.random.randn(N,100)
+    Param = 'max'
+    inputs = []
+    for cnt in range(N):
+        inputs.append((Data[cnt], Param))
+    someMul = RobustGaussianFittingLibrary.misc.multiprocessor(
+        myFunc, inputs, outputIsNumpy = True, showProgress = True)
+    means = np.squeeze(someMul())
+    print(np.array([ [means], [Data.max(1)]]).T)
+
+
 if __name__ == '__main__':
     print('PID ->' + str(os.getpid()))
+    test_getTriangularVertices()
+    test_fitBackgroundCylindrically()
+    test_fitValue2Skewed_sweep_over_N()
+    test_fitValue2SkewedSmallSample()
+    test_fitValue_sweep()
     test_fit2Poisson()
     test_medianOfFits()
     test_fitValueVSMeanShiftPy()
@@ -1153,9 +1280,7 @@ if __name__ == '__main__':
     test_fitBackgroundTensor()
     test_fitBackgroundTensor_multiproc()
     test_fitBackground()
-    test_fitValue_sweep()
     test_MSSE()
-    test_fitValue2Skewed_sweep_over_N()
     test_SginleGaussianVec()
     test_flatField()
     test_fitValueSmallSample()
@@ -1163,11 +1288,12 @@ if __name__ == '__main__':
     test_fitValue2Skewed()
     visOrderStat()
     test_removeIslands()
-    test_fitBackgroundRadially()
     test_fitLineTensor_MultiProc()
     test_textProgBar()
     test_bigTensor2SmallsInds()
     test_fitBackgroundRadiallyTensor_multiproc()
     test_PDF2Uniform()
     test_RobustAlgebraicLineFittingPy()
-    
\ No newline at end of file
+    test_multiprocessor()
+    print('This was robust fitting')
+    exit()
\ No newline at end of file
